/**************************************************************************************************
  Filename:       simplemeter.h
  Revised:        $Date: 2011-02-09 16:05:07 -0800 (Wed, 09 Feb 2011) $
  Revision:       $Revision: 25029 $

  Description:    Header file for the Simple Meter functionality


  Copyright 2009-2011 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
**************************************************************************************************/

#ifndef SIMPLEMETER_H
#define SIMPLEMETER_H

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************
 * INCLUDES
 */
#include "zcl.h"

/*********************************************************************
 * CONSTANTS
 */
#define SIMPLEMETER_ENDPOINT                 0x09

#define SIMPLEMETER_MAX_ATTRIBUTES           54

#define SIMPLEMETER_MAX_OPTIONS              2

#define SIMPLEMETER_UPDATE_TIME_PERIOD       1000   // Update time event in seconds
#define SIMPLEMETER_REPORT_PERIOD            5000   // reporting interval
#define SE_DEVICE_POLL_RATE                  8000   // Poll rate for SE end device

// Application Events
#define SIMPLEMETER_IDENTIFY_TIMEOUT_EVT           0x0001
#define SIMPLEMETER_UPDATE_TIME_EVT                0x0002
#define SIMPLEMETER_KEY_ESTABLISHMENT_REQUEST_EVT  0x0004
#define SIMPLEMETER_REPORT_ATTRIBUTE_EVT           0x0008

/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * TYPEDEFS
 */

/*********************************************************************
 * VARIABLES
 */
extern SimpleDescriptionFormat_t simpleMeterSimpleDesc;
extern CONST zclAttrRec_t simpleMeterAttrs[];
extern zclOptionRec_t simpleMeterOptions[];
extern uint8 simpleMeterDeviceEnabled;
extern uint16 simpleMeterTransitionTime;
extern uint16 simpleMeterIdentifyTime;
extern uint32 simpleMeterTime;

/*********************************************************************
 * FUNCTIONS
 */

 /*
  * Initialization for the task
  */
extern void simplemeter_Init( uint8 task_id );

/*
 *  Event Process for the task
 */
extern uint16 simplemeter_event_loop( uint8 task_id, uint16 events );


/*********************************************************************
*********************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* SIMPLEMETER_H */
