/******************************************************************************
  Filename:       GenericApp.c
  Revised:        $Date: 2010-12-21 10:27:34 -0800 (Tue, 21 Dec 2010) $
  Revision:       $Revision: 24670 $

  Description:    Generic Application (no Profile).


  Copyright 2004-2010 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product. Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
******************************************************************************/

/*********************************************************************
  This application isn't intended to do anything useful, it is
  intended to be a simple example of an application's structure.

  This application sends "Hello World" to another "Generic"
  application every 15 seconds.  The application will also
  receive "Hello World" packets.

  The "Hello World" messages are sent/received as MSG type message.

  This applications doesn't have a profile, so it handles everything
  directly - itself.

  Key control:
    SW1:
    SW2:  initiates end device binding
    SW3:
    SW4:  initiates a match description request
*********************************************************************/

/*********************************************************************
 * INCLUDES
 */
#include <string.h> // SCA


#include "OSAL.h"
#include "AF.h"
#include "ZDApp.h"
#include "ZDObject.h"
#include "ZDProfile.h"

#include "GenericApp.h"
#include "DebugTrace.h"

#if !defined( WIN32 )
  #include "OnBoard.h"
#endif

/* HAL */
#include "hal_lcd.h"
#include "hal_led.h"
#include "hal_key.h"
#include "hal_uart.h"

/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * CONSTANTS
 */

/*********************************************************************
 * FUNCTIONS
 */

void GenericApp_MessageMSGCBSCA(void);
void rellenarmensajeAenviar(afIncomingMSGPacket_t *pkt ,uint8 opcion);
void mostrarcomando(afIncomingMSGPacket_t *inpkt);
void GenericApp_SendTheMessageNEW(void);

/*********************************************************************
 * TYPEDEFS
 */

/*********************************************************************
 * GLOBAL VARIABLES
 */

uint8 mensajeAenviar32bytes[LONGITUD_TRAMA];
uint8 mensajeArecibir32bytes[LONGITUD_TRAMA];

zAddrType_t dstAddr;

// This list should be filled with Application specific Cluster IDs.
const cId_t GenericApp_ClusterList[GENERICAPP_MAX_CLUSTERS] =
{
  GENERICAPP_CLUSTERID
};

const SimpleDescriptionFormat_t GenericApp_SimpleDesc =
{
  GENERICAPP_ENDPOINT,              //  int Endpoint;
  GENERICAPP_PROFID,                //  uint16 AppProfId[2];
  GENERICAPP_DEVICEID,              //  uint16 AppDeviceId[2];
  GENERICAPP_DEVICE_VERSION,        //  int   AppDevVer:4;
  GENERICAPP_FLAGS,                 //  int   AppFlags:4;
  GENERICAPP_MAX_CLUSTERS,          //  byte  AppNumInClusters;
  (cId_t *)GenericApp_ClusterList,  //  byte *pAppInClusterList;
  GENERICAPP_MAX_CLUSTERS,          //  byte  AppNumInClusters;
  (cId_t *)GenericApp_ClusterList   //  byte *pAppInClusterList;
};

// This is the Endpoint/Interface description.  It is defined here, but
// filled-in in GenericApp_Init().  Another way to go would be to fill
// in the structure here and make it a "const" (in code space).  The
// way it's defined in this sample app it is define in RAM.
endPointDesc_t GenericApp_epDesc;

/*********************************************************************
 * EXTERNAL VARIABLES
 */
#if !defined( SERIAL_APP_PORT )
#define SERIAL_APP_PORT  0
#endif

#if !defined( SERIAL_APP_BAUD )
#define SERIAL_APP_BAUD  HAL_UART_BR_9600
//#define SERIAL_APP_BAUD  HAL_UART_BR_115200
#endif

// When the Rx buf space is less than this threshold, invoke the Rx callback.
#if !defined( SERIAL_APP_THRESH )
#define SERIAL_APP_THRESH  64
#endif

#if !defined( SERIAL_APP_RX_SZ )
#define SERIAL_APP_RX_SZ  128
#endif

#if !defined( SERIAL_APP_TX_SZ )
#define SERIAL_APP_TX_SZ  128
#endif

// Millisecs of idle time after a byte is received before invoking Rx callback.
#if !defined( SERIAL_APP_IDLE )
#define SERIAL_APP_IDLE  20 
#endif

// Loopback Rx bytes to Tx for throughput testing.
#if !defined( SERIAL_APP_LOOPBACK )
#define SERIAL_APP_LOOPBACK  FALSE
#endif

// This is the max byte count per OTA message.
#if !defined( SERIAL_APP_TX_MAX )
#define SERIAL_APP_TX_MAX  80
#endif

#define SERIAL_APP_RSP_CNT  4

/*********************************************************************
 * EXTERNAL FUNCTIONS
 */

/*********************************************************************
 * LOCAL VARIABLES
 */
byte GenericApp_TaskID;   // Task ID for internal task/event processing
                          // This variable will be received when
                          // GenericApp_Init() is called.
devStates_t GenericApp_NwkState;


byte GenericApp_TransID;  // This is the unique message ID (counter)

afAddrType_t GenericApp_DstAddr;

/*  B - SERIAL COMMUNICATION  */
/*  SCA not sure if we'll use this one  */


uint8 SerialApp_TaskID;    // Task ID for internal task/event processing.
static uint8 SerialApp_MsgID;

static afAddrType_t SerialApp_TxAddr;
static uint8 SerialApp_TxSeq;
static uint8 SerialApp_TxBuf[SERIAL_APP_TX_MAX+1];
static uint8 SerialApp_TxLen = 0;

static afAddrType_t SerialApp_RxAddr;
static uint8 SerialApp_RxSeq;
static uint8 SerialApp_RspBuf[SERIAL_APP_RSP_CNT];
/*********************************************************************
 * LOCAL FUNCTIONS
 */
void GenericApp_ProcessZDOMsgs( zdoIncomingMsg_t *inMsg );
void GenericApp_HandleKeys( byte shift, byte keys );
void GenericApp_MessageMSGCB( afIncomingMSGPacket_t *pckt );
void GenericApp_SendTheMessage( void );
void enviarmensajeUART(afIncomingMSGPacket_t *MSGpkt);

/*  B - SERIAL COMMUNICATION  */

static void SerialApp_ProcessZDOMsgs( zdoIncomingMsg_t *inMsg );
static void SerialApp_HandleKeys( uint8 shift, uint8 keys );
static void SerialApp_ProcessMSGCmd( afIncomingMSGPacket_t *pkt );
static void SerialApp_Send(void);
static void SerialApp_Resp(void);
static void SerialApp_CallBack(uint8 port, uint8 event);

/*********************************************************************
 * NETWORK LAYER CALLBACKS
 */

/*********************************************************************
 * PUBLIC FUNCTIONS
 */

/*********************************************************************
 * @fn      GenericApp_Init
 *
 * @brief   Initialization function for the Generic App Task.
 *          This is called during initialization and should contain
 *          any application specific initialization (ie. hardware
 *          initialization/setup, table initialization, power up
 *          notificaiton ... ).
 *
 * @param   task_id - the ID assigned by OSAL.  This ID should be
 *                    used to send messages and set timers.
 *
 * @return  none
 */
void GenericApp_Init( byte task_id )
{
  GenericApp_TaskID = task_id;
  GenericApp_NwkState = DEV_INIT;
  GenericApp_TransID = 0;

  // Device hardware initialization can be added here or in main() (Zmain.c).
  // If the hardware is application specific - add it here.
  // If the hardware is other parts of the device add it in main().

  //  GenericApp_DstAddr.addrMode = (afAddrMode_t)AddrNotPresent;
  GenericApp_DstAddr.addrMode = (afAddrMode_t)Addr16Bit;    //  Modified SCA
  //  GenericApp_DstAddr.endPoint = 0;
  GenericApp_DstAddr.endPoint  = GENERICAPP_ENDPOINT;
  //  GenericApp_DstAddr.addr.shortAddr = 0;
  GenericApp_DstAddr.addr.shortAddr = 0x0000;               //  Modified SCA  

  // Fill out the endpoint description.
  GenericApp_epDesc.endPoint = GENERICAPP_ENDPOINT;
  GenericApp_epDesc.task_id = &GenericApp_TaskID;
  GenericApp_epDesc.simpleDesc
            = (SimpleDescriptionFormat_t *)&GenericApp_SimpleDesc;
  GenericApp_epDesc.latencyReq = noLatencyReqs;

  // Register the endpoint description with the AF
  afRegister( &GenericApp_epDesc );

  // Register for all key events - This app will handle all key events
  RegisterForKeys( GenericApp_TaskID );

  // Update the display
#if defined ( LCD_SUPPORTED )
    HalLcdWriteString( "GenericApp", HAL_LCD_LINE_1 );
#endif

  ZDO_RegisterForZDOMsg( GenericApp_TaskID, End_Device_Bind_rsp );
  ZDO_RegisterForZDOMsg( GenericApp_TaskID, Match_Desc_rsp );
  
  //  added SCA  
   osal_start_timerEx( GenericApp_TaskID,
                                  GENERICAPP_SEND_MSG_EVT,
                                  GENERICAPP_SEND_MSG_TIMEOUT );
   
   osal_start_timerEx( GenericApp_TaskID,
                                  GENERICAPP_FINDDEVICES,
                                  GENERICAPP_RETRYLINKING  );
   
   
   /* B - SERIAL APP PART */ 
  
  halUARTCfg_t uartConfig;
  
  // SCA - maybe not needed
  SerialApp_TaskID = task_id;
  SerialApp_RxSeq = 0xC3;

  //afRegister( (endPointDesc_t *)&SerialApp_epDesc );    //  Register the SECOND Endpoint
                                                        //  as described previously
                                                        //  This EP has TWO clusters
  
  //  SCA already registered
  //  RegisterForKeys( task_id ); 

  uartConfig.configured           = TRUE;              // 2x30 don't care - see uart driver.
  uartConfig.baudRate             = SERIAL_APP_BAUD;
  uartConfig.flowControl          = TRUE;
  uartConfig.flowControlThreshold = SERIAL_APP_THRESH; // 2x30 don't care - see uart driver.
  uartConfig.rx.maxBufSize        = SERIAL_APP_RX_SZ;  // 2x30 don't care - see uart driver.
  uartConfig.tx.maxBufSize        = SERIAL_APP_TX_SZ;  // 2x30 don't care - see uart driver.
  uartConfig.idleTimeout          = SERIAL_APP_IDLE;   // 2x30 don't care - see uart driver.
  uartConfig.intEnable            = TRUE;              // 2x30 don't care - see uart driver.
  uartConfig.callBackFunc         = SerialApp_CallBack;
  HalUARTOpen (SERIAL_APP_PORT, &uartConfig);
  
  SerialApp_TxLen = 0;                                  //  lo inicializo aqu
  
  /*********************************************************************
 * @fn          ZDP_EndDeviceBindReq
 *
 * @brief       This builds and sends a End_Device_Bind_req message.
 *              This function sends a unicast message.
 *
 * @param       dstAddr - destination address
 * @param       LocalCoordinator - short address of local coordinator
 * @param       epIntf - Endpoint/Interface of Simple Desc
 * @param       ProfileID - Profile ID
 *
 *   The Input cluster list is the opposite of what you would think.
 *   This is the output cluster list of this device
 * @param       NumInClusters - number of input clusters
 * @param       InClusterList - input cluster ID list
 *
 *   The Output cluster list is the opposite of what you would think.
 *   This is the input cluster list of this device
 * @param       NumOutClusters - number of output clusters
 * @param       OutClusterList - output cluster ID list
 *
 * @param       SecurityEnable - Security Options
 *
 * @return      afStatus_t
 */
  
  
  
  
}

/*********************************************************************
 * @fn      GenericApp_ProcessEvent
 *
 * @brief   Generic Application Task event processor.  This function
 *          is called to process all events for the task.  Events
 *          include timers, messages and any other user defined events.
 *
 * @param   task_id  - The OSAL assigned task ID.
 * @param   events - events to process.  This is a bit map and can
 *                   contain more than one event.
 *
 * @return  none
 */
UINT16 GenericApp_ProcessEvent( byte task_id, UINT16 events )
{
  afIncomingMSGPacket_t *MSGpkt;
  afDataConfirm_t *afDataConfirm;

  // Data Confirmation message fields
  byte sentEP;
  ZStatus_t sentStatus;
  byte sentTransID;       // This should match the value sent
  (void)task_id;          // Intentionally unreferenced parameter

  if ( events & SYS_EVENT_MSG )
  {
    MSGpkt = (afIncomingMSGPacket_t *)osal_msg_receive( GenericApp_TaskID );
    while ( MSGpkt )
    {
      switch ( MSGpkt->hdr.event )
      {
        case ZDO_CB_MSG:
          GenericApp_ProcessZDOMsgs( (zdoIncomingMsg_t *)MSGpkt );
          break;

        case KEY_CHANGE:
          //  GenericApp_HandleKeys( ((keyChange_t *)MSGpkt)->state, ((keyChange_t *)MSGpkt)->keys );
          //  no se usan las teclas, que adems en produccin estarn deshabilitadas
          break;

        case AF_DATA_CONFIRM_CMD:
          // This message is received as a confirmation of a data packet sent.
          // The status is of ZStatus_t type [defined in ZComDef.h]
          // The message fields are defined in AF.h
          afDataConfirm = (afDataConfirm_t *)MSGpkt;
          sentEP = afDataConfirm->endpoint;
          sentStatus = afDataConfirm->hdr.status;
          sentTransID = afDataConfirm->transID;
          (void)sentEP;
          (void)sentTransID;

          // Action taken when confirmation is received.
          if ( sentStatus != ZSuccess )
          {
            // The data wasn't delivered -- Do something
          }
          break;

        case AF_INCOMING_MSG_CMD:
          
          
          //GenericApp_MessageMSGCB( MSGpkt );
#if HAL_LCD == TRUE  
          mostrarcomando(MSGpkt);
#endif
          
          //  rellenarmensajeAenviar(MSGpkt ,1);      //  No se devuelve mensaje OTA, al ser un
                                                      //  mero puente se delega en el dispositivo
                                                      //  conectado va serie la decisin de qu 
                                                      //  hacer
          enviarmensajeUART(MSGpkt);                  //  envia mensaje por uart
          break;

        case ZDO_STATE_CHANGE:
          GenericApp_NwkState = (devStates_t)(MSGpkt->hdr.status);
          /* if ( (GenericApp_NwkState == DEV_ZB_COORD)
              || (GenericApp_NwkState == DEV_ROUTER)
              || (GenericApp_NwkState == DEV_END_DEVICE) )
            {
            // Start sending "the" message in a regular interval.
            osal_start_timerEx( GenericApp_TaskID,
                                GENERICAPP_SEND_MSG_EVT,
                                GENERICAPP_SEND_MSG_TIMEOUT );
          } */
          
          //  SCA modification
          
#if MENSAJE_PERIODICO == TRUE         //  definido en GenericApp.h
          if(GenericApp_NwkState == DEV_END_DEVICE)
          {
              //SCA send every 5 sec ONLY if End Device
              osal_start_timerEx( GenericApp_TaskID,
                                  GENERICAPP_SEND_MSG_EVT,
                                  GENERICAPP_SEND_MSG_TIMEOUT );
          }
#endif
          break;

        default:
          break;
      }

      // Release the memory
      osal_msg_deallocate( (uint8 *)MSGpkt );

      // Next
      MSGpkt = (afIncomingMSGPacket_t *)osal_msg_receive( GenericApp_TaskID );
    }

    // return unprocessed events
    return (events ^ SYS_EVENT_MSG);
  }

  // Send a message out - This event is generated by a timer
  //  (setup in GenericApp_Init()).
  if ( events & GENERICAPP_SEND_MSG_EVT )
  {
    
#ifdef  MENSAJE_PERIODICO == TRUE
    GenericApp_SendTheMessageNEW();
#endif
  
    return (events ^ GENERICAPP_SEND_MSG_EVT);
  }
  
  
  if ( events & GENERICAPP_FINDDEVICES )
  {
    
      return (events ^ GENERICAPP_FINDDEVICES );
      
    }
  
  
  //}

  // Discard unknown events
  return 0;
}

/*********************************************************************
 * Event Generation Functions
 */

/*********************************************************************
 * @fn      GenericApp_ProcessZDOMsgs()
 *
 * @brief   Process response messages
 *
 * @param   none
 *
 * @return  none
 */
void GenericApp_ProcessZDOMsgs( zdoIncomingMsg_t *inMsg )
{
  switch ( inMsg->clusterID )
  {
    case End_Device_Bind_rsp:
      if ( ZDO_ParseBindRsp( inMsg ) == ZSuccess )
      {
        // Light LED
        HalLedSet( HAL_LED_4, HAL_LED_MODE_ON );
      }
#if defined( BLINK_LEDS )
      else
      {
        // Flash LED to show failure
        HalLedSet ( HAL_LED_4, HAL_LED_MODE_FLASH );
      }
#endif
      break;

    case Match_Desc_rsp:
      {
        ZDO_ActiveEndpointRsp_t *pRsp = ZDO_ParseEPListRsp( inMsg );
        if ( pRsp )
        {
          if ( pRsp->status == ZSuccess && pRsp->cnt )
          {
            GenericApp_DstAddr.addrMode = (afAddrMode_t)Addr16Bit;
            GenericApp_DstAddr.addr.shortAddr = pRsp->nwkAddr;
            // Take the first endpoint, Can be changed to search through endpoints
            GenericApp_DstAddr.endPoint = pRsp->epList[0];

            // Light LED
            HalLedSet( HAL_LED_4, HAL_LED_MODE_ON );
          }
          osal_mem_free( pRsp );
        }
      }
      break;
  }
}

/*********************************************************************
 * @fn      GenericApp_HandleKeys
 *
 * @brief   Handles all key events for this device.
 *
 * @param   shift - true if in shift/alt.
 * @param   keys - bit field for key events. Valid entries:
 *                 HAL_KEY_SW_4
 *                 HAL_KEY_SW_3
 *                 HAL_KEY_SW_2
 *                 HAL_KEY_SW_1
 *
 * @return  none
 */
void GenericApp_HandleKeys( byte shift, byte keys )
{
  // zAddrType_t dstAddr;

  // Shift is used to make each button/switch dual purpose.
  if ( shift )
  {
    if ( keys & HAL_KEY_SW_1 )
    { 
         HalLcdWriteString( "GenericApp", HAL_LCD_LINE_1 );
    }
    if ( keys & HAL_KEY_SW_2 )
    {
    }
    if ( keys & HAL_KEY_SW_3 )
    {
    }
    if ( keys & HAL_KEY_SW_4 )
    {
    }
    if ( keys & HAL_KEY_SW_6 )
    {
      HalLcdWriteString( "BUTTON_2_UP", HAL_LCD_LINE_1 );
    }
    
    
    
  }
  else
  {
    if ( keys & HAL_KEY_SW_1 )
    {
      HalLcdWriteString( "KEY_1_UP", HAL_LCD_LINE_1 );
      // Since SW1 isn't used for anything else in this application...
#if defined( SWITCH1_BIND )
      // we can use SW1 to simulate SW2 for devices that only have one switch,
      keys |= HAL_KEY_SW_2;
#elif defined( SWITCH1_MATCH )
      // or use SW1 to simulate SW4 for devices that only have one switch
      keys |= HAL_KEY_SW_4;
#endif
    }

    if ( keys & HAL_KEY_SW_2 )
    {
      HalLcdWriteString( "Binding...", HAL_LCD_LINE_1 );
      HalLedSet ( HAL_LED_4, HAL_LED_MODE_OFF );

      // Initiate an End Device Bind Request for the mandatory endpoint
      dstAddr.addrMode = Addr16Bit;
      dstAddr.addr.shortAddr = 0x0000; // Coordinator
      ZDP_EndDeviceBindReq( &dstAddr, NLME_GetShortAddr(),
                            GenericApp_epDesc.endPoint,
                            GENERICAPP_PROFID,
                            GENERICAPP_MAX_CLUSTERS, (cId_t *)GenericApp_ClusterList,
                            GENERICAPP_MAX_CLUSTERS, (cId_t *)GenericApp_ClusterList,
                            FALSE );
    }

    if ( keys & HAL_KEY_SW_3 )
    {
    }

    if ( keys & HAL_KEY_SW_4 )
    {
      HalLedSet ( HAL_LED_4, HAL_LED_MODE_OFF );
      // Initiate a Match Description Request (Service Discovery)
      dstAddr.addrMode = AddrBroadcast;
      dstAddr.addr.shortAddr = NWK_BROADCAST_SHORTADDR;
      ZDP_MatchDescReq( &dstAddr, NWK_BROADCAST_SHORTADDR,
                        GENERICAPP_PROFID,
                        GENERICAPP_MAX_CLUSTERS, (cId_t *)GenericApp_ClusterList,
                        GENERICAPP_MAX_CLUSTERS, (cId_t *)GenericApp_ClusterList,
                        FALSE );
    }
    
    if ( keys & HAL_KEY_SW_5 )
    {
      HalLcdWriteString( "J_PRESS_DOWN", HAL_LCD_LINE_1 );
      HalLedSet ( HAL_LED_1, HAL_LED_MODE_ON );
    }
    
    if ( keys & HAL_KEY_SW_6 )
    {
      HalLcdWriteString( "BUTTON_2_UP", HAL_LCD_LINE_1 );
      HalLedSet ( HAL_LED_2, HAL_LED_MODE_ON );
    }
    
  }
}

/*********************************************************************
 * LOCAL FUNCTIONS
 */

/*********************************************************************
 * @fn      GenericApp_MessageMSGCB
 *
 * @brief   Data message processor callback.  This function processes
 *          any incoming data - probably from other devices.  So, based
 *          on cluster ID, perform the intended action.
 *
 * @param   none
 *
 * @return  none
 */
void GenericApp_MessageMSGCB( afIncomingMSGPacket_t *pkt )
{
  switch ( pkt->clusterId )
  {
    case GENERICAPP_CLUSTERID:
    // "the" message
#if defined( LCD_SUPPORTED )
      //  HalLcdWriteScreen( (char*)pkt->cmd.Data, "rcvd" ); //SCA (line1, line2)
      
/**************************************************************************************************
 * @fn      HalLcdWriteStringValue
 *
 * @brief   Write a string followed by a value to the LCD
 *
 * @param   title  - Title that will be displayed before the value
 *          value  - value
 *          format - redix
 *          line   - line number
 *
 * @return  None
 **************************************************************************************************/
      HalLcdWriteString((char*)pkt->cmd.Data , 1 );
      HalLcdWriteStringValue("Origen: ", pkt->srcAddr.addr.shortAddr, 16, 2 );
      
      /*  ONLY if this is the coordinator   */
      char theMessageDataResponse[] = "COORD_AWARE!";
      if ( (GenericApp_NwkState == DEV_ZB_COORD) )
      {
            GenericApp_DstAddr.addr.shortAddr = pkt->srcAddr.addr.shortAddr;    //  SCA change the destination address 
            AF_DataRequest( &GenericApp_DstAddr, &GenericApp_epDesc,
                       GENERICAPP_CLUSTERID,
                       (byte)osal_strlen( theMessageDataResponse ) + 1,
                       (byte *)&theMessageDataResponse,
                       &GenericApp_TransID,
                       AF_DISCV_ROUTE, AF_DEFAULT_RADIUS );  
        
        
      }
      
#elif defined( WIN32 )
      WPRINTSTR( pkt->cmd.Data );
#endif
      break;
  }
}

/*********************************************************************
 * @fn      GenericApp_SendTheMessage
 *
 * @brief   Send "the" message.
 *
 * @param   none
 *
 * @return  none
 */
void GenericApp_SendTheMessage( void )
{
  char theMessageData[] = "ENDDEV_HERE!";
  
  /*********************************************************************
 * @fn      AF_DataRequest
 *
 * @brief   Common functionality for invoking APSDE_DataReq() for both
 *          SendMulti and MSG-Send.
 *
 * input parameters
 *
 * @param  *dstAddr - Full ZB destination address: Nwk Addr + End Point.
 * @param  *srcEP - Origination (i.e. respond to or ack to) End Point Descr.
 * @param   cID - A valid cluster ID as specified by the Profile.
 * @param   len - Number of bytes of data pointed to by next param.
 * @param  *buf - A pointer to the data bytes to send.
 * @param  *transID - A pointer to a byte which can be modified and which will
 *                    be used as the transaction sequence number of the msg.
 * @param   options - Valid bit mask of Tx options.
 * @param   radius - Normally set to AF_DEFAULT_RADIUS.
 *
 * output parameters
 *
 * @param  *transID - Incremented by one if the return value is success.
 *
 * @return  afStatus_t - See previous definition of afStatus_... types.
 */
  if ( (GenericApp_NwkState == DEV_END_DEVICE) )
  {
    HalLcdWriteString( "Mensaje Enviado", HAL_LCD_LINE_2 );
    if ( AF_DataRequest( &GenericApp_DstAddr, &GenericApp_epDesc,
                        GENERICAPP_CLUSTERID,
                        (byte)osal_strlen( theMessageData ) + 1,
                        (byte *)&theMessageData,
                        &GenericApp_TransID,
                        AF_DISCV_ROUTE, AF_DEFAULT_RADIUS ) == afStatus_SUCCESS )
    {
      // Successfully requested to be sent.
    }
    else
    {
      // Error occurred in request to send.
    }
  }
}

/*********************************************************************
 * @fn      SerialApp_CallBack
 *
 * @brief   Send data OTA.
 *
 * @param   port - UART port.
 * @param   event - the UART port event flag.
 *
 * @return  none
 */
static void SerialApp_CallBack(uint8 port, uint8 event)
{
  
  
  (void)port;
  /*****************************************************************************
 * @fn      HalUARTRead
 *
 * @brief   Read a buffer from the UART
 *
 * @param   port - USART module designation
 *          buf  - valid data buffer at least 'len' bytes in size
 *          len  - max length number of bytes to copy to 'buf'
 *
 * @return  length of buffer that was read
 *****************************************************************************/
//  uint16 HalUARTRead(uint8 port, uint8 *buf, uint16 len)
  
  //  start writing only IF SerialApp_TxLen = 0 and the incoming byte is 0
  
  
   //SerialApp_TxLen += HalUARTRead(SERIAL_APP_PORT, SerialApp_TxBuf + SerialApp_TxLen,1);
   //HalUARTRead(SERIAL_APP_PORT, SerialApp_TxBuf + SerialApp_TxLen,1);
   //SerialApp_TxLen++;
  
   HalUARTRead(SERIAL_APP_PORT, SerialApp_TxBuf, LONGITUD_TRAMA);
   HalLedSet (HAL_LED_4, HAL_LED_MODE_TOGGLE);  //  Para verificar que se ha recibido un byte desde el
                                                //  equipo, en este caso el ordenador
   // significa que ya podemos enviar
   //if((SerialApp_TxLen == (LONGITUD_TRAMA - 1)) && (SerialApp_TxBuf[0] == 0xFF) && (SerialApp_TxBuf[31] == '\0'))
   if((SerialApp_TxBuf[0] == 0xFF) && (SerialApp_TxBuf[31] == '\0'))
   {
          //  fill in the right buffer
          for(int i = 0;i < LONGITUD_TRAMA;i++){mensajeAenviar32bytes[i] = SerialApp_TxBuf[i];}
          
          //  clean the buffer to prevent echoes
          
          for(int i = 0; i < LONGITUD_TRAMA; i++){ SerialApp_TxBuf[i] = 0xAA ;}
            
          // send the mensajeAenviar32bytes
          
          GenericApp_MessageMSGCBSCA();
        
          //  counter to zero
         
          HalLedSet (HAL_LED_3, HAL_LED_MODE_TOGGLE);
   }
   
  
  
 
}



//  Esta funcin no se usa en esta implementacin
void rellenarmensajeAenviar(afIncomingMSGPacket_t *pkt ,uint8 opcion)
{
  
  /* typedef struct{
  
    //  0     0xFF
    
    //  1-2   origin node address  
    uint8* originnode2bytes;
    //  3     EP origin
    uint8* endpointorigin1byte;
    //  4-5   destination node address
    uint8* destinationnode2bytes;  
    //  6     EP destination
    uint8* endpointdestination1byte;
    //  7     Destination cluster
    uint8*  destinationcluster1byte;
    //  8-9 16-bit command
    uint8* command2bytes;
    //  10-31 21 bytes of data
    uint8*  bytesofdata21;
    
  } framedata_t; */
      
  switch(opcion)
  {  
      case 1:
            //  inicio de la trama/frame
            mensajeAenviar32bytes[0] = 0xFF;
            mensajeAenviar32bytes[1] = (uint8)(NLME_GetShortAddr());                 //  LSB   
            mensajeAenviar32bytes[2] = (uint8)(NLME_GetShortAddr() >> 8);            //  MSB
            //  Origin EP
            mensajeAenviar32bytes[3] = GENERICAPP_ENDPOINT;
            
            //  destination node address LSB
            mensajeAenviar32bytes[4] =(uint8) (pkt->srcAddr.addr.shortAddr);          //  ESTO LA HACE BIEN
            //  destination node address MSB
            mensajeAenviar32bytes[5] =(uint8) ((pkt->srcAddr.addr.shortAddr) >> 8);   //  ESTO LO HACE BIEN  
            
            //  Destination EP
            mensajeAenviar32bytes[6] = GENERICAPP_ENDPOINT;
            
            //  Destination cluster (in this case I dont use clusters)
            mensajeAenviar32bytes[7] = 0;     //  not relevant in this application
            
            //if ( (GenericApp_NwkState == DEV_ZB_COORD) )
            //{  //  command LSB
            //  mensajeAenviar32bytes[8] = 0x51;  //  como prueba
              //  command MSB
            //  mensajeAenviar32bytes[9] = 0x53;  //  como prueba
            //  }
            if ( (GenericApp_NwkState == DEV_END_DEVICE) )
            {
              
              mensajeAenviar32bytes[8] =  (uint8) 0x7B; 
              mensajeAenviar32bytes[9] =  (uint8) 0x7D;
            }
            //  data fields (10 - 31) not used now
            
        break;
        
      case 2:
        
        break;
        
      case 3:
      
        break;
      
      default:

        break;
          
  }     
        GenericApp_MessageMSGCBSCA();
        
}
  
 
void GenericApp_MessageMSGCBSCA(void)
{
 
/**************************************************************************************************
 * @fn      HalLcdWriteStringValue
 *
 * @brief   Write a string followed by a value to the LCD
 *
 * @param   title  - Title that will be displayed before the value
 *          value  - value
 *          format - redix
 *          line   - line number
 *
 * @return  None
 **************************************************************************************************/
      //  HalLcdWriteString((char*)pkt->cmd.Data , 1 );
      //  HalLcdWriteStringValue("Origen: ", pkt->srcAddr.addr.shortAddr, 16, 2 );
  
      /*********************************************************************
 * @fn      AF_DataRequest
 *
 * @brief   Common functionality for invoking APSDE_DataReq() for both
 *          SendMulti and MSG-Send.
 *
 * input parameters
 *
 * @param  *dstAddr - Full ZB destination address: Nwk Addr + End Point.
 * @param  *srcEP - Origination (i.e. respond to or ack to) End Point Descr.
 * @param   cID - A valid cluster ID as specified by the Profile.
 * @param   len - Number of bytes of data pointed to by next param.
 * @param  *buf - A pointer to the data bytes to send.
 * @param  *transID - A pointer to a byte which can be modified and which will
 *                    be used as the transaction sequence number of the msg.
 * @param   options - Valid bit mask of Tx options.
 * @param   radius - Normally set to AF_DEFAULT_RADIUS.
 *
 * output parameters
 *
 * @param  *transID - Incremented by one if the return value is success.
 *
 * @return  afStatus_t - See previous definition of afStatus_... types.
 */
  
/* typedef struct{
  
    //  0     0xFF
    
    //  1-2   origin node address  
    uint8* originnode2bytes;
    //  3     EP origin
    uint8* endpointorigin1byte;
    //  4-5   destination node address
    uint8* destinationnode2bytes;  
    //  6     EP destination
    uint8* endpointdestination1byte;
    //  7     Destination cluster
    uint8*  destinationcluster1byte;
    //  8-9 16-bit command
    uint8* command2bytes;
    //  10-31 21 bytes of data
    uint8*  bytesofdata21;
    
  } framedata_t; */  
  
  
      /*  ONLY if this is the coordinator   */
      // char theMessageDataResponse[] = "COORD_AWARE!";
      //if ( (GenericApp_NwkState == DEV_ZB_COORD) )
      //{
            //  GenericApp_DstAddr.addr.shortAddr = pkt->srcAddr.addr.shortAddr;          //  SCA change the destination address 
        
            GenericApp_DstAddr.addr.shortAddr = (uint16)(mensajeAenviar32bytes[4] & 0x00ff) | ((mensajeAenviar32bytes[5] << 8) & 0xff00 );        //  ESTO TB LO HACE BIEN
            //  A donde enva ...
            HalLcdWriteStringValue("Destino: ", GenericApp_DstAddr.addr.shortAddr,16,3);  //  ESTO TB LO HACE BIEN
            //  adems debe rellenar los campos de ORIGEN (direccin de origen y EP)
            
            mensajeAenviar32bytes[1] = (uint8)(NLME_GetShortAddr());                 //  LSB   
            mensajeAenviar32bytes[2] = (uint8)(NLME_GetShortAddr() >> 8);            //  MSB
            //  Origin EP
            mensajeAenviar32bytes[3] = GENERICAPP_ENDPOINT;                          //  EP
            
            
            //  Orden de envio ...
            AF_DataRequest( &GenericApp_DstAddr, &GenericApp_epDesc,
                       GENERICAPP_CLUSTERID,
                       LONGITUD_TRAMA,
                       (byte *)&mensajeAenviar32bytes,
                       &GenericApp_TransID,
                       AF_DISCV_ROUTE, AF_DEFAULT_RADIUS );  
            
           // limpiamos el buffer de envio para evitar ecos 
        
            for(int i = 0; i < LONGITUD_TRAMA; i++){mensajeAenviar32bytes[i] = 0xAA;}
        
      //}
}



void mostrarcomando(afIncomingMSGPacket_t *inpkt)
{
     
    //  uint8 message[32];
    //  message = &(inpkt->cmd.Data[4]);
    
    //uint16 lectura = (uint16)((inpkt->cmd.Data[4]) & 0x00ff) | (((inpkt->cmd.Data[5])  << 8) & 0xff00 );
  if ( (GenericApp_NwkState == DEV_END_DEVICE) ){ HalLedSet (HAL_LED_2, HAL_LED_MODE_TOGGLE);}
  else{HalLedSet (HAL_LED_1, HAL_LED_MODE_TOGGLE);}
    
  
    HalLcdWriteStringValue("Origen: ", (uint16)((inpkt->cmd.Data[4]) & 0x00ff) | (((inpkt->cmd.Data[5])  << 8) & 0xff00 ), 16,2);
    HalLcdWriteStringValue("Comando: ", (uint16)((inpkt->cmd.Data[8]) & 0x00ff) | (((inpkt->cmd.Data[9])  << 8) & 0xff00 ), 16,1);
    
  
    //  HalLcdWriteStringValue("Length: ", (uint16)inpkt->cmd.DataLength, 10, 2);       //  ESTE FUNCIONA BIEN
 
}


void GenericApp_SendTheMessageNEW()
{

 //   Inicio de la trama/frame
 //   Este mensaje es de identificacin
            mensajeAenviar32bytes[0] = 0xFF;
             
            mensajeAenviar32bytes[1] = (uint8)(NLME_GetShortAddr());                 //  LSB   
            mensajeAenviar32bytes[2] = (uint8)(NLME_GetShortAddr() >> 8);            //  MSB
            //  Origin EP
            mensajeAenviar32bytes[3] = GENERICAPP_ENDPOINT;
            
            //  destination node address LSB
            mensajeAenviar32bytes[4] =(uint8) 0x00;          //   ENVIAR CADA 30s a Coordinator
            //  destination node address MSB
            mensajeAenviar32bytes[5] =(uint8) 0x00;           //  ENVIAR CADA 30s a Coordinator  
            
            //  Destination EP
            mensajeAenviar32bytes[6] = GENERICAPP_ENDPOINT;
            
            //  Destination cluster (in this case I dont use clusters)
            mensajeAenviar32bytes[7] = 0;     //  not relevant in this application
            
            
            //  command LSB
            mensajeAenviar32bytes[8] = 0x00;    //  version definitiva
            //  command MSB
            mensajeAenviar32bytes[9] = 0x00;     //  version definitiva
            
            
            //  data fields (10 - 31) 
            mensajeAenviar32bytes[10] = 'M' ;
            mensajeAenviar32bytes[11] = 'e';
            mensajeAenviar32bytes[12] = 'n';
            mensajeAenviar32bytes[13] = 's';
            mensajeAenviar32bytes[14] = 'a';
            mensajeAenviar32bytes[15] = 'j';
            mensajeAenviar32bytes[16] = 'e';
            mensajeAenviar32bytes[17] = ' ';
            mensajeAenviar32bytes[18] = 'p';
            mensajeAenviar32bytes[19] = 'e';
            mensajeAenviar32bytes[20] = 'r';
            mensajeAenviar32bytes[21] = 'i';
            mensajeAenviar32bytes[22] = 'o';
            mensajeAenviar32bytes[23] = 'd';
            mensajeAenviar32bytes[24] = 'i';
            mensajeAenviar32bytes[25] = 'c';
            mensajeAenviar32bytes[26] = 'o';
            mensajeAenviar32bytes[27] = '.';
            mensajeAenviar32bytes[28] = '.';
            mensajeAenviar32bytes[29] = '.';
            mensajeAenviar32bytes[30] = '.';
            
            
            
            mensajeAenviar32bytes[31] = '\0';
            
#if MENSAJE_PERIODICO == TRUE            
            //  Enviar solo si EndDevice
             if ( (GenericApp_NwkState == DEV_END_DEVICE) )
             {
                        GenericApp_MessageMSGCBSCA();
                         //   si se enva se reenva otra vez, de esta forma
                         //   se tiene un mensaje que se enva desde este ENDDEVICE
                         //   (en el caso de que lo sea) al COORDINATOR
                        osal_start_timerEx( GenericApp_TaskID,
                                  GENERICAPP_SEND_MSG_EVT,
                                  GENERICAPP_SEND_MSG_TIMEOUT );
             }

#endif
             
}



void enviarmensajeUART(afIncomingMSGPacket_t *pkt)
{
        if(pkt->cmd.DataLength == LONGITUD_TRAMA)  
        {
              HalLedSet (HAL_LED_3, HAL_LED_MODE_TOGGLE);
              HalUARTWrite(SERIAL_APP_PORT, pkt->cmd.Data, LONGITUD_TRAMA);
             
              
        }

}

