//	Project specific includes
#include "writeinserial.h"

//	UNIX and Standard C includes
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <sys/syscall.h>
#include <sys/types.h>






// 	FUNCION
//	Rellena el buffer variable global a enviar serie	
void rellenaBuffer(void)
{
	
	mensajeaenviar[0]  = (char) 0xFF;	// el primero SIEMPRE ES 0xFF
	mensajeaenviar[1]  = (char) 0x00;	// dir. ORIGEN LSB - lo rellena el SoC	
	mensajeaenviar[2]  = (char) 0x00;	// dir. ORIGEN MSB - lo rellena el SoC
	mensajeaenviar[3]  = (char) 0x00;	// EP.  ORIGEN - lo rellena el SoC
	mensajeaenviar[4]  = (char) 0xEC;	// dir. DESTINO LSB - SE RELLENA AQUÍ
	mensajeaenviar[5]  = (char) 0x56;	// dir. DESTINO MSB - SE RELLENA AQUÍ
	mensajeaenviar[6]  = (char) 0x0A;	// EP.  DESTINO - SE RELLENA AQUÍ
	mensajeaenviar[7]  = (char) 0x00;	// Cluster. DESTINO - no se usa
	mensajeaenviar[8]  = (char) 0xAA;	// Comando LSB
	mensajeaenviar[9]  = (char) 0xEE;	// Comando MSB
	mensajeaenviar[10] = (char)'E';		// Datos (aquí codificamos ASCII ?)
	mensajeaenviar[11] = (char)'n';
	mensajeaenviar[12] = (char)'v';
	mensajeaenviar[13] = (char)'i';
	mensajeaenviar[14] = (char)'a';
	mensajeaenviar[15] = (char)'d';
	mensajeaenviar[16] = (char)'o';
	mensajeaenviar[17] = (char)' ';
	mensajeaenviar[18] = (char)'d';
	mensajeaenviar[19] = (char)'e';
	mensajeaenviar[20] = (char)'s';
	mensajeaenviar[21] = (char)'d';
	mensajeaenviar[22] = (char)'e';
	mensajeaenviar[23] = (char)' ';
	mensajeaenviar[24] = (char)'P';
	mensajeaenviar[25] = (char)'C';
	mensajeaenviar[26] = (char)' ';
	mensajeaenviar[27] = (char)' ';
	mensajeaenviar[28] = (char)' ';
	mensajeaenviar[29] = (char)' ';
	mensajeaenviar[30] = (char)' ';
	mensajeaenviar[31] = (char)'\0';  	// el último siempre es '\0'
}

//	FUNCION
//	opens the PUERTOSERIEOBJETIVO defined as a chain in writeinserial.h
int open_port(void)
{	
	int fd;
	


	//	El puerto serie se especifica al arrancar el diálogo
	fd = open(entrada_dispositivo, O_RDWR | O_NOCTTY | O_NDELAY);

	if(fd == -1)
	{
		//	Could not open the port
			perror("open_port: unable to open the port ");
	
	}
	else
	{
		fcntl(fd, F_SETFL, 0);
		return(fd);
	}

}


//	FUNCION
//	opens the PUERTOSERIEOBJETIVO defined as a chain in writeinserial.h
//	blocking function
int open_port_bloqueante(void)
{
	int fd;
	


	//	El puerto serie se especifica al arrancar el diálogo
	fd = open(entrada_dispositivo, O_RDWR );

	if(fd == -1)
	{
		//	Could not open the port
			perror("open_port: unable to open the port ");
	
	}
	else
	{
		fcntl(fd, F_SETFL, 0);
		return(fd);
	}


}


// 	FUNCION
//	Rellena el buffer variable global a enviar serie CON LOS DATOS SUMINISTRADOS 
void rellenaBufferConInfo(void)
{


	//	Local variable
	int i;


	direcciondestino_LSB = (char) ( entrada_direcciondestino & 0x000000FF );
	direcciondestino_MSB = (char) ((entrada_direcciondestino >> 8) & 0x000000FF );
	endpointdestino = (char) ( entrada_endpointdestino & 0x000000FF );
	clusterdestino = (char) ( entrada_clusterdestino & 0x000000FF );
	comando_LSB = (char) ( entrada_comando & 0x000000FF);
	comando_MSB = (char) ((entrada_comando >> 8) & 0x000000FF);



	mensajeaenviar[0]  = (char) 0xFF;	// el primero SIEMPRE ES 0xFF
	mensajeaenviar[1]  = (char) 0x00;	// dir. ORIGEN LSB - lo rellena el SoC	
	mensajeaenviar[2]  = (char) 0x00;	// dir. ORIGEN MSB - lo rellena el SoC
	mensajeaenviar[3]  = (char) 0x00;	// EP.  ORIGEN - lo rellena el SoC
	mensajeaenviar[4]  = direcciondestino_LSB;	// dir. DESTINO LSB - SE RELLENA AQUÍ
	mensajeaenviar[5]  = direcciondestino_MSB;	// dir. DESTINO MSB - SE RELLENA AQUÍ
	mensajeaenviar[6]  = endpointdestino;		// EP.  DESTINO - SE RELLENA AQUÍ
	mensajeaenviar[7]  = clusterdestino;		// Cluster. DESTINO - no se usa
	mensajeaenviar[8]  = (char) comando_LSB;	// Comando LSB
	mensajeaenviar[9]  = (char) comando_MSB;	// Comando MSB

	for(i = 10;i < (10 + LONGITUDDATOS + 1); i++)
	{
		mensajeaenviar[i] = entrada_datos[i - 10]; 
	}

	mensajeaenviar[31] = '\0';

}



// 	FUNCION
//	Imprime los datos interpretados ANTES de abrir el puerto 
void verificaloqueseenvia(void)
{
	int i;
	for(i = 0; i < LONGITUDBUFFERSERIE;i++)
	{	
		printf("byte en posicion %2.2i, valor %2.2x expresado en hexadecimal\n", i ,mensajeaenviar[i]);
	}

}





















