//	Project specific includes
#include "writeinserial.h"

//	UNIX and Standard C includes
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <sys/syscall.h>
#include <sys/types.h>


/*	generate the dynamic libraries (so objects)
	
	gcc -Wall -fPIC -c functionswriteinserial.c escribiryenviar.c
	gcc -shared -Wl, -soname, libzigbeeti.so.1 -o libzigbeeti.so.1.0 functionswriteinserial.o escribiryenviar.o

	en el directorio donde se ponga, hay que crear DOS symbolic links

	ln -sf /path-to-library-directory/libzigbeeti.so.1.0 /path-to-link-directory/libzigbeeti.so
	ln -sf /path-to-library-directory/libzigbeeti.so.1.0 /path-to-link-directory/libzigbeeti.so.1	

*/



//	FUNCION
//	cada vez que se invoca devuelve UNA trama de LONGITUDBUFFERSERIE octetos
//	así que cada vez que para recibir constantemente hay que meter esta función
//	en alguna estructura iterativa
//	puertoserie		cadena del tipo /dev/ttyUSB0"
//	pmensajearecibir	puntero de tipo unsigned char* con LONGITUDBUFFERSERIE bytes reservados
//				en la llamada.	
void zigbeeti_recibir(char* puertoserie, unsigned char* pmensajearecibir)
{
	
	//	Variables locales
	int fd;		//	Port handler
	int nbytes;	//	no bytes enviados

	//	se abre el puerto
	fd = zigbeeti_open_port_bloqueante(puertoserie);
	read(fd, pmensajearecibir , 1);				// POSIX

	//	comprueba que el primer byte de la trama es 0xFF
	while(*pmensajearecibir != 0xFF){read(fd, pmensajearecibir , 1);}
	nbytes = 1;
	
	for(nbytes = 1; nbytes < LONGITUDBUFFERSERIE; nbytes++)
	{
		read(fd , pmensajearecibir + nbytes, 1 );	// POSIX
	}	
}






//	FUNCION
//	se usa para generar una trama de 32 octetos a enviar, suministrando la siguiente información
//	unsigned int direcciondestino	--	dirección de destino que se convertirá en 2 bytes
//	unsigned int endpointdestino	--	endpoint (1 byte) de destino
//	unsigned int clusterdestino	--	cluster de destino, en caso de usarse (1 byte)
//	unsigned int comando		--	comando (2 bytes)
//	char* cadenaasciidatos		--	puntero a un string que encapsula los datos a enviar
//	car* puertoserie		--	puntero a un string que describa el puerto a abrir
void zigbeeti_enviar(unsigned int direcciondestino, unsigned int endpointdestino, unsigned int clusterdestino, unsigned int comando, char* cadenaasciidatos,char* puertoserie)
{
	
	//	local variables
	//int i;
	int fd;								//	Port handler
	int nbytes;							//	no bytes enviados
	//unsigned char mensajeaenviar[LONGITUDBUFFERSERIE];
	//unsigned char pmensajeaenviar[LONGITUDBUFFERSERIE];		//	inicializado como array para reservar LONGITUDBUFFERSERIE
	unsigned char pmensajeaenviar[LONGITUDBUFFERSERIE];									//	bytes	

	//	rellena el buffer apuntado por pmensajea enviar
		
	zigbeeti_rellenaBufferConInfo(direcciondestino , endpointdestino , clusterdestino , comando ,   cadenaasciidatos, pmensajeaenviar);

	//for(i = 0; i < LONGITUDBUFFERSERIE; i++)
	//{
	//	mensajeaenviar[i] = *(pmensajeaenviar + i);
	//}
	//	verifica el tratamiento de los datos	
	zigbeeti_verificaloqueseenvia(pmensajeaenviar);
	//	se abre el puerto
	fd = zigbeeti_open_port_bloqueante(puertoserie);
	//	se escribe en el puerto
	// nbytes = write(fd, &mensajeaenviar[0], LONGITUDBUFFERSERIE);	// POSIX
	nbytes = write(fd , pmensajeaenviar , LONGITUDBUFFERSERIE);
	//	se cierra el puerto
	close(fd);							// POSIX
}

